import UIKit
import GoogleMobileAds

enum NativeAdmobType: String {
    case full, row, banner
}

class NativeAdView: GADUnifiedNativeAdView {
    
    let adLabelLbl: UILabel = {
        let label = UILabel()
        label.font = UIFont.systemFont(ofSize: 10)
        label.textColor = UIColor.init(rgb: 0x55C3E7)
        label.text = "Ad"
        label.numberOfLines = 1
        label.textAlignment = .center
        return label
    }()
    
    lazy var adLabelView: UIView = {
        let view = UIView()
        view.backgroundColor = UIColor.init(rgb: 0xFFFFFF)
        view.layer.backgroundColor = UIColor.init(rgb: 0xFFFFFF).cgColor
        view.layer.cornerRadius = 2
        view.layer.borderWidth = 1
        view.layer.borderColor = UIColor.init(rgb: 0x55C3E7).cgColor
        view.clipsToBounds = true
        view.addSubview(adLabelLbl)
        view.autoSetDimension(.width, toSize: 20)
        view.autoSetDimension(.height, toSize: 16)
        adLabelLbl.autoPinEdgesToSuperviewEdges(with: UIEdgeInsets(top: 0, left: 3, bottom: 0, right: 3))
        
        return view
    }()
    
    let adMediaView = GADMediaView()
    
    let adIconView: UIImageView!
    
    let adHeadLineLbl: UILabel = {
        let label = UILabel()
        label.font = UIFont.boldSystemFont(ofSize: 13)
        label.numberOfLines = 1
        label.sizeToFit()
        return label
    }()
    
    let adAdvertiserLbl: UILabel = {
        let label = UILabel()
        label.font = UIFont.systemFont(ofSize: 10)
        label.numberOfLines = 1
        return label
    }()
    
    let adRatingView = StackLayout().spacing(2)
    
    let adBodyLbl: UILabel = {
        let label = UILabel()
        label.font = UIFont.systemFont(ofSize: 11)
        label.numberOfLines = 3
        label.sizeToFit()
        return label
    }()

    let adPriceLbl: UILabel = {
        let label = UILabel()
        label.font = UIFont.systemFont(ofSize: 11)
        label.numberOfLines = 1
        label.textAlignment = .right
        return label
    }()

    let adStoreLbl: UILabel = {
        let label = UILabel()
        label.font = UIFont.systemFont(ofSize: 11)
        label.numberOfLines = 1
        label.textAlignment = .right
        return label
    }()

    let callToActionBtn: UIButton = {
        let button = UIButton()
        button.setBackgroundImage(.from(color: .fromHex("#5A7BEF")), for: .normal)
        button.setTitleColor(.white, for: .normal)
        button.titleLabel?.font = UIFont.systemFont(ofSize: 11)
        return button
    }()

    var starIcon: StarIcon {
        let icon = StarIcon()
        icon.autoSetDimensions(to: CGSize(width: 13, height: 13))
        return icon
    }

    var options = NativeAdmobOptions() {
        didSet { updateOptions() }
    }

    let type: NativeAdmobType

    init(frame: CGRect, type: NativeAdmobType) {
        self.type = type

        switch type {
        case .full:
            adIconView = {
                   let imageView = UIImageView()
                   imageView.autoSetDimensions(to: CGSize(width: 40, height: 40))
                   return imageView
               }()
            adIconView.layer.borderColor = UIColor.init(rgb: 0xDDDDDD).cgColor
            break;
        case .row:
            adIconView = {
               let imageView = UIImageView()
               imageView.autoSetDimensions(to: CGSize(width: 72, height: 72))
               return imageView
            }()
            adIconView.layer.borderColor = UIColor.init(rgb: 0xDDDDDD).cgColor
            break;
        case .banner:
            adIconView = {
               let imageView = UIImageView()
               imageView.autoSetDimensions(to: CGSize(width: 44, height: 44))
               return imageView
            }()
            adIconView.layer.borderColor = UIColor.init(rgb: 0xDDDDDD).cgColor
            break;
        }


        super.init(frame: frame)
        setupView()
    }

    required init?(coder: NSCoder) {
        fatalError("No support for interface builder")
    }

    func setNativeAd(_ nativeAd: GADUnifiedNativeAd?) {
        guard let nativeAd = nativeAd else { return }
        self.nativeAd = nativeAd

        // Set the mediaContent on the GADMediaView to populate it with available
        // video/image asset.
        adMediaView.mediaContent = nativeAd.mediaContent

        // Populate the native ad view with the native ad assets.
        // The headline is guaranteed to be present in every native ad.
        adHeadLineLbl.text = nativeAd.headline

        adAdvertiserLbl.text = nativeAd.advertiser
        if nativeAd.advertiser.isNilOrEmpty {
            adAdvertiserLbl.isHidden = true
        }

        // These assets are not guaranteed to be present. Check that they are before
        // showing or hiding them.
        adBodyLbl.text = nativeAd.body
        if nativeAd.body.isNilOrEmpty {
            adBodyLbl.isHidden = true
        }

        callToActionBtn.setTitle(nativeAd.callToAction, for: .normal)
        if nativeAd.callToAction.isNilOrEmpty {
            callToActionBtn.isHidden = true
        }

        adIconView.image = nativeAd.icon?.image
        adIconView.isHidden = nativeAd.icon == nil

        adRatingView.arrangedSubviews.forEach { view in
            view.removeFromSuperview()
        }
        let numOfStars = Int(truncating: nativeAd.starRating ?? 0)
        adRatingView.children(Array(0..<numOfStars).map { _ in
            let icon = self.starIcon
            icon.color = options.ratingColor
            return icon
        })
        adRatingView.isHidden = nativeAd.starRating == nil

        adStoreLbl.text = nativeAd.store
        if nativeAd.store.isNilOrEmpty {
            adStoreLbl.isHidden = true
        }

        adPriceLbl.text = nativeAd.price
        if nativeAd.price.isNilOrEmpty {
            adPriceLbl.isHidden = true
        }

        layoutIfNeeded()
    }
}

private extension NativeAdView {

    func setupView() {
        self.mediaView = adMediaView
        self.headlineView = adHeadLineLbl
        self.callToActionView = callToActionBtn
        self.iconView = adIconView
        self.bodyView = adBodyLbl
        self.storeView = adStoreLbl
        self.priceView = adPriceLbl
        self.starRatingView = adRatingView
        self.advertiserView = adAdvertiserLbl

        do {
            setupLayout()
        } catch {
            print("failed setupLayout error: \(error.localizedDescription)")
        }
    }


    func setupLayout() {
        switch type {
        case .full: setupFullLayout()
        case .row: setupRowLayout()
        case .banner: setupBannerLayout()
        }
    }

    func setupFullLayout() {
        adBodyLbl.numberOfLines = 3
        adBodyLbl.sizeToFit()

        adIconView.layer.cornerRadius = 20
        adIconView.layer.masksToBounds = true
        adIconView.autoSetDimensions(to: CGSize(width: 40, height: 40))

        callToActionBtn.layer.cornerRadius = 4
        callToActionBtn.layer.masksToBounds = true
        callToActionBtn.autoSetDimension(.width, toSize: 100)
        callToActionBtn.autoSetDimension(.height, toSize: 26)

        let infoLayout = StackLayout().spacing(16).children([
            adIconView,
            StackLayout().direction(.vertical).children([
                adHeadLineLbl,
                StackLayout().children([
                    StackLayout().spacing(6).direction(.horizontal).children([
                        adLabelView,
                        StackLayout().spacing(0).direction(.vertical).children([
                            adAdvertiserLbl,
                            adRatingView,
                            UIView()
                        ]),
                    ]),
                    UIView()
                ]),
            ]),
        ])

        callToActionBtn.setContentHuggingPriority(.defaultHigh, for: .horizontal)

        adLabelLbl.autoSetDimension(.height, toSize: 15, relation: .greaterThanOrEqual)

        let topView = UIView()
        addSubview(topView)
        topView.autoPinEdge(toSuperviewEdge: .top)
        topView.autoPinEdge(toSuperviewEdge: .leading, withInset: 0, relation: .greaterThanOrEqual)

        let contentView = UIView()
        addSubview(contentView)
        contentView.autoPinEdgesToSuperviewEdges(with: .zero, excludingEdge: .top)
        contentView.autoPinEdge(.top, to: .top, of: topView, withOffset: 0)

        let mediaLayout = StackLayout().direction(.horizontal).spacing(0).alignItems(UIStackView.Alignment.top).children([
            adMediaView,
        ])

        let bodyLayout = StackLayout().direction(.horizontal).spacing(0).alignItems(UIStackView.Alignment.top).children([
            adBodyLbl,
        ])

        let buttonLayout = StackLayout().direction(.horizontal).spacing(0).alignItems(UIStackView.Alignment.top).children([
            callToActionBtn,
        ])

        let layout = StackLayout().direction(.vertical).spacing(0).alignItems(UIStackView.Alignment.leading).children([
                mediaLayout,
                infoLayout,
                bodyLayout,
                buttonLayout,
            ])

        mediaLayout.isLayoutMarginsRelativeArrangement = true
        mediaLayout.layoutMargins = UIEdgeInsets(top: 4, left: 0, bottom: 0, right: 0)

        infoLayout.isLayoutMarginsRelativeArrangement = true
        infoLayout.layoutMargins = UIEdgeInsets(top: 10, left: 10, bottom: 0, right: 10)

        bodyLayout.isLayoutMarginsRelativeArrangement = true
        bodyLayout.layoutMargins = UIEdgeInsets(top: 10, left: 10, bottom: 0, right: 10)

        buttonLayout.isLayoutMarginsRelativeArrangement = true
        buttonLayout.layoutMargins = UIEdgeInsets(top: 10, left: 10, bottom: 10, right: 10)
        buttonLayout.autoPinEdge(toSuperviewEdge: .leading)
        buttonLayout.autoPinEdge(toSuperviewEdge: .trailing)

//        layout.isUserInteractionEnabled = false
        contentView.addSubview(layout)
        layout.alignment = UIStackView.Alignment.top
        layout.distribution = UIStackView.Distribution.fill
        layout.autoAlignAxis(toSuperviewAxis: .horizontal)
        layout.autoPinEdge(toSuperviewEdge: .leading)
        layout.autoPinEdge(toSuperviewEdge: .trailing)
        layout.autoPinEdge(toSuperviewEdge: .bottom)
    }

    func setupRowLayout() {
        adBodyLbl.numberOfLines = 3
        adBodyLbl.sizeToFit()

        adIconView.layer.cornerRadius = 4
        adIconView.layer.masksToBounds = true
        adIconView.autoSetDimensions(to: CGSize(width: 72, height: 72))

        callToActionBtn.layer.cornerRadius = 4
        callToActionBtn.layer.masksToBounds = true
        callToActionBtn.autoSetDimension(.width, toSize: 100)
        callToActionBtn.autoSetDimension(.height, toSize: 26)
        callToActionBtn.setContentHuggingPriority(.defaultHigh, for: .horizontal)

        let topView = UIView()
        addSubview(topView)
        topView.autoPinEdge(toSuperviewEdge: .top, withInset: 0, relation: .greaterThanOrEqual)
        topView.autoPinEdge(toSuperviewEdge: .leading, withInset: 0, relation: .greaterThanOrEqual)

        let contentView = UIView()
        addSubview(contentView)
        contentView.autoPinEdgesToSuperviewEdges(with: .zero, excludingEdge: .top)
        contentView.autoPinEdge(.top, to: .bottom, of: topView, withOffset: 0)

        let bodyLayout = StackLayout().spacing(6).direction(.vertical).children([
            StackLayout().alignItems(UIStackView.Alignment.leading).direction(.horizontal).children([
                adHeadLineLbl,
            ]),
            StackLayout().alignItems(UIStackView.Alignment.leading).direction(.horizontal).children([
                adBodyLbl,
            ]),
            UIView(),
            StackLayout().direction(.horizontal).children([
                StackLayout().alignItems(UIStackView.Alignment.center).direction(.vertical).children([
                    UIView(),
                    StackLayout().spacing(6).alignItems(UIStackView.Alignment.leading).direction(.horizontal).children([
                        adLabelView,
                        StackLayout().spacing(0).direction(.vertical).children([
                            adAdvertiserLbl,
                            adRatingView,
                            UIView()
                        ]),
                    ]),
                    UIView(),
                ]),
                UIView(),
                callToActionBtn,
            ]),
        ])

        let layout = StackLayout().spacing(10).alignItems(UIStackView.Alignment.leading).direction(.horizontal).children([
            adIconView,
            bodyLayout,
        ])

        bodyLayout.isLayoutMarginsRelativeArrangement = true

        layout.isLayoutMarginsRelativeArrangement = true
        layout.layoutMargins = UIEdgeInsets(top: 0, left: 12, bottom: 0, right: 12)
        layout.isUserInteractionEnabled = false

        contentView.addSubview(layout)
        layout.autoAlignAxis(toSuperviewAxis: .horizontal)
        layout.autoPinEdge(toSuperviewEdge: .leading)
        layout.autoPinEdge(toSuperviewEdge: .trailing)
        layout.autoPinEdge(toSuperviewEdge: .top, withInset: 0, relation: .greaterThanOrEqual)
        layout.autoPinEdge(toSuperviewEdge: .bottom, withInset: 0, relation: .greaterThanOrEqual)
    }

    func setupBannerLayout() {
        adBodyLbl.numberOfLines = 3
        
        adIconView.layer.cornerRadius = 22
        adIconView.layer.masksToBounds = true
        adIconView.autoSetDimensions(to: CGSize(width: 44, height: 44))
        
        callToActionBtn.layer.cornerRadius = 4
        callToActionBtn.layer.masksToBounds = true
        callToActionBtn.autoSetDimension(.height, toSize: 26)
        callToActionBtn.setContentHuggingPriority(.defaultHigh, for: .horizontal)
        
        adLabelLbl.autoSetDimension(.height, toSize: 15, relation: .greaterThanOrEqual)
        
        let topView = UIView()
        addSubview(topView)
        topView.autoPinEdge(toSuperviewEdge: .top, withInset: 0, relation: .greaterThanOrEqual)
        topView.autoPinEdge(toSuperviewEdge: .leading, withInset: 0, relation: .greaterThanOrEqual)

        let contentView = UIView()
        addSubview(contentView)
        contentView.autoPinEdgesToSuperviewEdges(with: .zero, excludingEdge: .top)
        contentView.autoPinEdge(.top, to: .bottom, of: topView, withOffset: 0)
        
        let layout = StackLayout().spacing(6).direction(.vertical).children([
            StackLayout().spacing(16).direction(.horizontal).children([
                adIconView,
                StackLayout().direction(.vertical).children([
                    adHeadLineLbl,
                    StackLayout().spacing(4).direction(.horizontal).children([
                        adLabelView,
                        StackLayout().children([
                            adAdvertiserLbl,
                            adRatingView,
                            UIView()
                        ]),
                    ]),
                ]),
            ]),
            callToActionBtn,
        ])
        
        layout.isUserInteractionEnabled = false
        
        contentView.addSubview(layout)
        layout.autoAlignAxis(toSuperviewAxis: .horizontal)
        layout.autoPinEdge(toSuperviewEdge: .leading, withInset: 16, relation: .lessThanOrEqual)
        layout.autoPinEdge(toSuperviewEdge: .trailing, withInset: 16, relation: .lessThanOrEqual)
        layout.autoPinEdge(toSuperviewEdge: .top, withInset: 0, relation: .greaterThanOrEqual)
    }
    
    func updateOptions() {
        adMediaView.isHidden = !options.showMediaContent
        adMediaView.autoSetDimension(.height, toSize: options.sizeMediaContent)
        
        adLabelLbl.textColor = options.adLabelTextStyle.color
        adLabelLbl.font = UIFont.systemFont(ofSize: options.adLabelTextStyle.fontSize)
        adLabelView.backgroundColor = options.adLabelTextStyle.backgroundColor ?? .fromHex("FFFFFF")
        adLabelView.isHidden = !options.adLabelTextStyle.isVisible
        
        adHeadLineLbl.textColor = options.headlineTextStyle.color
        adHeadLineLbl.font = UIFont.systemFont(ofSize: options.headlineTextStyle.fontSize)
        adHeadLineLbl.isHidden = !options.headlineTextStyle.isVisible
        
        adAdvertiserLbl.textColor = options.advertiserTextStyle.color
        adAdvertiserLbl.font = UIFont.systemFont(ofSize: options.advertiserTextStyle.fontSize)
        adAdvertiserLbl.isHidden = !options.advertiserTextStyle.isVisible
        
        adBodyLbl.textColor = options.bodyTextStyle.color
        adBodyLbl.font = UIFont.systemFont(ofSize: options.bodyTextStyle.fontSize)
        adBodyLbl.isHidden = !options.bodyTextStyle.isVisible
        
        adStoreLbl.textColor = options.storeTextStyle.color
        adStoreLbl.font = UIFont.systemFont(ofSize: options.storeTextStyle.fontSize)
        adStoreLbl.isHidden = !options.storeTextStyle.isVisible
        
        adPriceLbl.textColor = options.priceTextStyle.color
        adPriceLbl.font = UIFont.systemFont(ofSize: options.priceTextStyle.fontSize)
        adPriceLbl.isHidden = !options.priceTextStyle.isVisible
        
        callToActionBtn.setTitleColor(options.callToActionStyle.color, for: .normal)
        callToActionBtn.titleLabel?.font = UIFont.systemFont(ofSize: options.callToActionStyle.fontSize)
        if let bgColor = options.callToActionStyle.backgroundColor {
            callToActionBtn.setBackgroundImage(.from(color: bgColor), for: .normal)
        }
        callToActionBtn.isHidden = !options.callToActionStyle.isVisible
        
        starIcon.color = options.ratingColor
    }
}


extension UIColor {
   convenience init(red: Int, green: Int, blue: Int) {
       assert(red >= 0 && red <= 255, "Invalid red component")
       assert(green >= 0 && green <= 255, "Invalid green component")
       assert(blue >= 0 && blue <= 255, "Invalid blue component")

       self.init(red: CGFloat(red) / 255.0, green: CGFloat(green) / 255.0, blue: CGFloat(blue) / 255.0, alpha: 1.0)
   }

   convenience init(rgb: Int) {
       self.init(
           red: (rgb >> 16) & 0xFF,
           green: (rgb >> 8) & 0xFF,
           blue: rgb & 0xFF
       )
   }
}
