import UIKit

class StackLayout: UIStackView {
    
    init() {
        super.init(frame: .zero)
        setupView()
    }
    
    required init(coder: NSCoder) {
        super.init(coder: coder)
        setupView()
    }
    
    func setupView() {}
}

extension StackLayout {
    
    @discardableResult
    func direction(_ axis: NSLayoutConstraint.Axis) -> StackLayout {
        self.axis = axis
        return self
    }
    
    @discardableResult
    func alignItems(_ alignment: Alignment) -> StackLayout {
        self.alignment = alignment
        return self
    }
    
    @discardableResult
    func justifyContent(_ distribution: Distribution) -> StackLayout {
        self.distribution = distribution
        return self
    }
    
    @discardableResult
    func spacing(_ value: CGFloat) -> StackLayout {
        spacing = value
        return self
    }
    
    @discardableResult
    func children(_ children: [Any]) -> StackLayout {
        children.forEach { child in
            if let view = getView(for: child) {
                addArrangedSubview(view)
            }
        }
        return self
    }
    
    @discardableResult
    func child(_ child: Any, at index: Int) -> StackLayout {
        guard index >= 0 && index < arrangedSubviews.count else { return self }
        
        if let view = getView(for: child) {
            insertArrangedSubview(view, at: index)
        }
        
        return self
    }
    
    @discardableResult
    func removeChild(at index: Int) -> StackLayout {
        guard index >= 0 && index < arrangedSubviews.count else { return self }
        
        let child = arrangedSubviews[index]
        removeArrangedSubview(child)
        child.removeFromSuperview()
        
        return self
    }
    
    private func getView(for child: Any) -> UIView? {
        if let stackItem = child as? StackItem {
            return stackItem.build(with: self)
        } else if let view = child as? UIView {
            return view
        }
        
        return nil
    }
}

protocol StackItem {
    func build(with layout: StackLayout) -> UIView
}

class StackViewItem: UIView, StackItem {
    
    enum Attribute {
        case leading(insets: UIEdgeInsets)
        
        case trailing(insets: UIEdgeInsets)
        
        case top(insets: UIEdgeInsets)
        
        case bottom(insets: UIEdgeInsets)
        
        case center(insets: UIEdgeInsets)
        
        case fill(insets: UIEdgeInsets)
    }
    
    private let originalView: UIView
    private let constraintsDefinition: ((UIView) -> ())

    init(view: UIView, constraintsDefinition: @escaping ((UIView) -> ())) {
        self.originalView = view
        self.constraintsDefinition = constraintsDefinition
        super.init(frame: .zero)
    }
    
    init(view: UIView, attribute: Attribute) {
        self.originalView = view
        self.constraintsDefinition = { (view) in
            switch attribute {
            case .leading(let insets):
                view.autoPinEdgesToSuperviewEdges(with: insets, excludingEdge: .trailing)
                view.autoPinEdge(toSuperviewEdge: .trailing, withInset: insets.right, relation: .greaterThanOrEqual)
                
            case .trailing(let insets):
                view.autoPinEdgesToSuperviewEdges(with: insets, excludingEdge: .leading)
                view.autoPinEdge(toSuperviewEdge: .leading, withInset: insets.left, relation: .greaterThanOrEqual)
                
            case .top(let insets):
                view.autoPinEdgesToSuperviewEdges(with: insets, excludingEdge: .bottom)
                view.autoPinEdge(toSuperviewEdge: .bottom, withInset: insets.bottom, relation: .greaterThanOrEqual)
                
            case .bottom(let insets):
                view.autoPinEdgesToSuperviewEdges(with: insets, excludingEdge: .top)
                view.autoPinEdge(toSuperviewEdge: .top, withInset: insets.top, relation: .greaterThanOrEqual)
                
            case .center(let insets):
                view.autoCenterInSuperview()
                view.autoPinEdge(toSuperviewEdge: .top, withInset: insets.top, relation: .greaterThanOrEqual)
                view.autoPinEdge(toSuperviewEdge: .leading, withInset: insets.left, relation: .greaterThanOrEqual)
                view.autoPinEdge(toSuperviewEdge: .bottom, withInset: insets.bottom, relation: .greaterThanOrEqual)
                view.autoPinEdge(toSuperviewEdge: .trailing, withInset: insets.right, relation: .greaterThanOrEqual)
                
            case .fill(let insets):
                view.autoPinEdgesToSuperviewEdges(with: insets)
            }
        }
        super.init(frame: .zero)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func build(with layout: StackLayout) -> UIView {
        addSubview(originalView)
        constraintsDefinition(originalView)
        return self
    }
}

struct StackSpaceItem: StackItem {
    
    private let width: CGFloat
    private let height: CGFloat
    
    init(size: CGFloat) {
        width = size
        height = size
    }
    
    init(width: CGFloat) {
        self.width = width
        self.height = 0
    }
    
    init(height: CGFloat) {
        self.width = 0
        self.height = height
    }
    
    func build(with layout: StackLayout) -> UIView {
        let view = UIView()
        view.autoSetDimensions(to: CGSize(width: width, height: height))
        
        return view
    }
}
