import 'dart:async';

import 'package:admob_network_ad/native_admob_controller.dart';
import 'package:admob_network_ad/native_admob_options.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';


const _viewType = "native_admob";

enum NativeAdmobType { banner, full, row }

class NativeAdmob extends StatefulWidget {
  final String adUnitID;
  final double height;
  final NativeAdmobOptions options;
  final NativeAdmobType type;

  final Widget loading;
  final Widget error;

  final NativeAdmobController controller;

  NativeAdmob({
    Key key,
    @required this.adUnitID,
    this.height,
    this.options,
    this.loading,
    this.error,
    this.controller,
    this.type = NativeAdmobType.full,
  })  : assert(adUnitID.isNotEmpty),
        super(key: key);

  @override
  _NativeAdmobState createState() => _NativeAdmobState();
}

class _NativeAdmobState extends State<NativeAdmob> {
  static final isAndroid = defaultTargetPlatform == TargetPlatform.android;
  static final isiOS = defaultTargetPlatform == TargetPlatform.iOS;

  NativeAdmobController _nativeAdController;

  NativeAdmobOptions get _options => widget.options ?? NativeAdmobOptions();
  NativeAdmobType get _type => widget.type ?? NativeAdmobType.full;

  Widget get _loading =>
      widget.loading ?? Container(
        height: widget.height,
        child: Center(
          child: CircularProgressIndicator(
            valueColor: AlwaysStoppedAnimation<Color>(Colors.blueGrey),
          ),
        ),
      );

  Widget get _error => widget.error ?? Container();

  var _loadState = AdLoadState.loading;
  StreamSubscription _subscription;
  double opacityHeight;

  @override
  void initState() {
    super.initState();
    _nativeAdController = widget.controller ?? NativeAdmobController();
    _nativeAdController.setAdUnitID(widget.adUnitID);

    _subscription = _nativeAdController.stateChanged.listen((state) {
      setState(() {
        _loadState = state;
      });
    });

    opacityHeight = widget.height;
  }

  @override
  void dispose() {
    _subscription.cancel();

    // We only dispose internal controller, external controller will be kept
    if (widget.controller == null) _nativeAdController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (isAndroid || isiOS) {
      return Stack(
        children: <Widget>[
          Container(
            child: _loadState == AdLoadState.loading
                ? _loading
                : _loadState == AdLoadState.loadError
                ? _error
                : Container(
              height: widget.height,
              margin: EdgeInsets.all(1.0),
              child: _createPlatformView(),
            ),
          ),
          AnimatedOpacity(
            opacity: (_loadState == AdLoadState.loadCompleted) ? 0 : 1,
            duration: Duration(milliseconds: 1000),
            curve: Curves.easeInBack,
            onEnd: () {
              if (_loadState == AdLoadState.loadCompleted) {
                opacityHeight = 0.0;
              } else {
                opacityHeight = widget.height;
              }
              setState(() {});
            },
            child: Container(
              height: opacityHeight,
              color: Colors.white,
            ),
          )
        ],
      );
    }

    return Text('$defaultTargetPlatform is not supported PlatformView yet.');
  }

  Widget _createPlatformView() {
    final creationParams = {
      "options": _options.toJson(),
      "controllerID": _nativeAdController.id,
      "type": _type.toString().replaceAll("NativeAdmobType.", ""),
    };

    return isAndroid
        ? AndroidView(
      viewType: _viewType,
      creationParamsCodec: StandardMessageCodec(),
      creationParams: creationParams,
    )
        : UiKitView(
      viewType: _viewType,
      creationParamsCodec: StandardMessageCodec(),
      creationParams: creationParams,
    );
  }
}
